unit Tiles;

interface

uses
  Graphics, SysUtils, Dialogs;

const
  TileWidth = 64;
  TileHeight = 64;
type
  TFileTile = record
    Bitmap: array[0..TileWidth - 1, 1..TileHeight - 1] of TColor;
    Freemap: array[0..TileWidth - 1, 1..TileHeight - 1] of Byte;
  end;

type
  TTile = record
    Bitmap: TBitmap;
    Freemap: TBitmap;
  end;

type
  TTiles = class
  private
    FTiles: array of TTile;
    FTileCount: Byte;
  public
    constructor Create;
    destructor Destroy; override;
    function LoadTiles(FileName: String): Boolean;
    function GetTile(Index: Byte): TBitmap;
    function TileCount: Byte;
  end;

implementation

{ TTiles }

constructor TTiles.Create;
begin
  FTileCount := 0;
end;

destructor TTiles.Destroy;
var
  I: Integer;
begin
  for I := 1 to FTileCount do
  begin
    FTiles[I-1].Bitmap.Free;
    FreeAndNil(FTiles[I-1].Bitmap);
    FTiles[I-1].FreeMap.Free;
    FreeAndNil(FTiles[I-1].Freemap);
  end;
  inherited Destroy;
end;

function TTiles.GetTile(Index: Byte): TBitmap;
begin
  if Index < FTileCount then
    Result := FTiles[Index].Bitmap
  else
    Result := nil;
end;

function TTiles.LoadTiles(FileName: String): Boolean;
var
  F: file of TFileTile;
  FT: TFileTile;
  I, J: Integer;
begin
  if not FileExists(FileName) then
  begin
    ShowMessage('Error during resource loading: File not found (' + FileName + ')');
    Result := False;
    Exit;
  end;

  AssignFile(F, FileName);
  Reset(F);
  while not Eof(F) do
  try
    Read(F, FT);
    Inc(FTileCount);
    SetLength(FTiles, FTileCount);
    FTiles[FTileCount - 1].Bitmap.Create;
    FTiles[FTileCount - 1].Bitmap.PixelFormat := pf32bit;
    FTiles[FTileCount - 1].Bitmap.Width := TileWidth;
    FTiles[FTileCount - 1].Bitmap.Height := TileHeight;
    for I := 0 to TileWidth - 1 do
      for J := 0 to TileHeight - 1 do
        FTiles[FTileCount - 1].Bitmap.Canvas.Pixels[I, J] := FT.Bitmap[I, J];
  except
    on E: Exception do
    begin
      ShowMessage('Error during resource loading: ' + E.Message);
      Result := False;
      Exit;
      break;
    end;
  end;
  Result := True;
end;

function TTiles.TileCount: Byte;
begin
  Result := FTileCount;
end;

end.
